@@ -27,11 +27,8 @@
 #include <string.h>
 
 #include <glib/gi18n.h>
+#include <gio/gio.h>
 #include <gdk/gdkkeysyms.h>
-#include <libgnomevfs/gnome-vfs-result.h>
-#include <libgnomevfs/gnome-vfs-uri.h>
-#include <libgnomevfs/gnome-vfs-utils.h>
-#include <libgnomevfs/gnome-vfs-xfer.h>
 #include <gconf/gconf-client.h>
 
 #include "launcher.h"
@@ -52,6 +49,7 @@
 typedef struct {
 	GtkWidget    *pixmap;
 	const char   *stock_id;
+	GIcon        *gicon;
 	char         *image;
 	char         *fallback_image;
 	GtkIconTheme *icon_theme;
@@ -78,6 +76,7 @@ static GtkWidget *populate_menu_from_directory (GtkWidget          *menu,
 static void panel_load_menu_image_deferred (GtkWidget   *image_menu_item,
 					    GtkIconSize  icon_size,
 					    const char  *stock_id,
+					    GIcon       *gicon,
 					    const char  *image_filename,
 					    const char  *fallback_image_filename);
 
@@ -311,7 +310,14 @@ icon_to_load_free (IconToLoad *icon)
 	if (!icon)
 		return;
 
-	g_object_unref (icon->pixmap); icon->pixmap = NULL;
+	if (icon->pixmap)
+		g_object_unref (icon->pixmap);
+	icon->pixmap = NULL;
+
+	if (icon->gicon)
+		g_object_unref (icon->gicon);
+	icon->gicon = NULL;
+
 	g_free (icon->image);          icon->image = NULL;
 	g_free (icon->fallback_image); icon->fallback_image = NULL;
 	g_free (icon);
@@ -328,6 +334,10 @@ icon_to_load_copy (IconToLoad *icon)
 	retval = g_new0 (IconToLoad, 1);
 
 	retval->pixmap         = g_object_ref (icon->pixmap);
+	if (icon->gicon)
+		retval->gicon  = g_object_ref (icon->gicon);
+	else
+		retval->gicon  = NULL;
 	retval->image          = g_strdup (icon->image);
 	retval->fallback_image = g_strdup (icon->fallback_image);
 	retval->stock_id       = icon->stock_id;
@@ -385,8 +395,27 @@ panel_make_menu_icon (GtkIconTheme *icon_theme,
 	}
 
 	if (pb == NULL) {
-		pb = gdk_pixbuf_new_from_file_at_size (file, size, size, NULL);
-		
+		pb = gdk_pixbuf_new_from_file (file, NULL);
+		if (pb) {
+			gint width, height;
+
+			width = gdk_pixbuf_get_width (pb);
+			height = gdk_pixbuf_get_height (pb);
+			
+			/* if we want 24 and we get 22, do nothing;
+			 * else scale */
+			if (!(size - 2 <= width && width <= size &&
+                              size - 2 <= height && height <= size)) {
+				GdkPixbuf *tmp;
+
+				tmp = gdk_pixbuf_scale_simple (pb, size, size,
+							       GDK_INTERP_BILINEAR);
+
+				g_object_unref (pb);
+				pb = tmp;
+			}
+		}
+				
 		/* add icon to the hash table so we don't load it again */
 		loaded = TRUE;
 	}
@@ -548,6 +577,54 @@ load_icons_handler_again:
 		icon_to_add->pixbuf    = NULL;
 		icon_to_add->icon_size = icon->icon_size;
 
+		icons_to_add = g_list_prepend (icons_to_add, icon_to_add);
+	} else if (icon->gicon) {
+		IconToAdd *icon_to_add;
+		char      *icon_name;
+		GdkPixbuf *pb;
+		int        icon_height = PANEL_DEFAULT_MENU_ICON_SIZE;
+
+		gtk_icon_size_lookup (icon->icon_size, NULL, &icon_height);
+
+		icon_name = panel_util_get_icon_name_from_g_icon (icon->gicon);
+
+		if (icon_name) {
+			pb = panel_make_menu_icon (icon->icon_theme,
+						   icon_name,
+						   icon->fallback_image,
+						   icon_height,
+						   &long_operation);
+			g_free (icon_name);
+		} else {
+			pb = panel_util_get_pixbuf_from_g_loadable_icon (icon->gicon, icon_height);
+			if (!pb && icon->fallback_image) {
+				pb = panel_make_menu_icon (icon->icon_theme,
+							   NULL,
+							   icon->fallback_image,
+							   icon_height,
+							   &long_operation);
+			}
+		}
+
+		if (!pb) {
+			icon_to_load_free (icon);
+			if (long_operation)
+				/* this may have been a long operation so jump
+				 * back to the main loop for a while */
+				return TRUE;
+			else
+				/* we didn't do anything long/hard, so just do
+				 * this again, this is fun, don't go back to
+				 * main loop */
+				goto load_icons_handler_again;
+		}
+
+		icon_to_add            = g_new (IconToAdd, 1);
+		icon_to_add->image     = g_object_ref (icon->pixmap);
+		icon_to_add->stock_id  = NULL;
+		icon_to_add->pixbuf    = pb;
+		icon_to_add->icon_size = icon->icon_size;
+
 		icons_to_add = g_list_prepend (icons_to_add, icon_to_add);
 	} else {
 		IconToAdd *icon_to_add;
@@ -618,20 +695,16 @@ static void
 add_app_to_desktop (GtkWidget      *item,
 		    GMenuTreeEntry *entry)
 {
-	GnomeVFSURI *vfs_source_uri;
-	GnomeVFSURI *vfs_target_uri;
-	GnomeVFSResult res;
-	char *source_uri;
+	GFile *source;
+	GFile *target;
+	GError *error;
 	char *target_dir;
 	char *target_uri;
+	char *source_uri;
 
 	g_return_if_fail (entry != NULL);
 
-	source_uri = g_filename_to_uri (gmenu_tree_entry_get_desktop_file_path (entry), NULL, NULL);
-	g_return_if_fail (source_uri != NULL);
-
-	vfs_source_uri = gnome_vfs_uri_new (source_uri);
-	g_return_if_fail (vfs_source_uri != NULL);
+	source = g_file_new_for_path (gmenu_tree_entry_get_desktop_file_path (entry));
 
 	if (desktop_is_home_dir ()) {
 		target_dir = g_build_filename (g_get_home_dir (), NULL);
@@ -639,29 +712,27 @@ add_app_to_desktop (GtkWidget      *item,
 		target_dir = g_strdup (g_get_user_special_dir (G_USER_DIRECTORY_DESKTOP));
 	}
 
+	source_uri = g_file_get_uri (source);
 	target_uri = panel_make_unique_desktop_uri (target_dir, source_uri);
-	g_return_if_fail (target_uri != NULL);
-
-	vfs_target_uri = gnome_vfs_uri_new (target_uri);
-	g_return_if_fail (vfs_target_uri != NULL);
-
-	res = gnome_vfs_xfer_uri (vfs_source_uri, vfs_target_uri,
-				  GNOME_VFS_XFER_DEFAULT,
-				  GNOME_VFS_XFER_ERROR_MODE_ABORT,
-				  GNOME_VFS_XFER_OVERWRITE_MODE_ABORT,
-				  NULL, NULL);
-
-	if (res != GNOME_VFS_OK) {
-		g_warning ("Problem while copying launcher to desktop: %s",
-			   gnome_vfs_result_to_string (res));
-	}
-
-	gnome_vfs_uri_unref (vfs_source_uri);
-	gnome_vfs_uri_unref (vfs_target_uri);
-
 	g_free (source_uri);
 	g_free (target_dir);
+	g_return_if_fail (target_uri != NULL);
+
+	target = g_file_new_for_uri (target_uri);
 	g_free (target_uri);
+
+	error = NULL;
+	g_file_copy (source, target, G_FILE_COPY_NONE,
+		     NULL, NULL, NULL, &error);
+
+	g_object_unref (source);
+	g_object_unref (target);
+	
+	if (error != NULL) {
+		g_warning ("Problem while copying launcher to desktop: %s",
+			   error->message);
+		g_error_free (error);
+	}
 }
 
 
@@ -1108,7 +1179,7 @@ drag_data_get_menu_cb (GtkWidget        *widget,
 	char       *uri_list;
 
 	path = gmenu_tree_entry_get_desktop_file_path (entry);
-	uri = gnome_vfs_get_uri_from_local_path (path);
+	uri = g_filename_to_uri (path, NULL, NULL);
 	uri_list = g_strconcat (uri, "\r\n", NULL);
 	g_free (uri);
 
@@ -1403,7 +1474,7 @@ create_submenu_entry (GtkWidget          *menu,
 
 	panel_load_menu_image_deferred (menuitem,
 					panel_menu_icon_get_size (),
-					NULL,
+					NULL, NULL,
 					gmenu_tree_directory_get_icon (directory),
 					PANEL_ICON_FOLDER);
 
@@ -1481,7 +1552,7 @@ create_menuitem (GtkWidget          *menu,
 
 	panel_load_menu_image_deferred (menuitem,
 					panel_menu_icon_get_size (),
-					NULL,
+					NULL, NULL,
 					alias_directory ? gmenu_tree_directory_get_icon (alias_directory) :
 							  gmenu_tree_entry_get_icon (entry),
 					NULL);
@@ -1718,11 +1789,13 @@ setup_menu_item_with_icon (GtkWidget   *item,
 			   GtkIconSize  icon_size,
 			   const char  *icon_name,
 			   const char  *stock_id,
+			   GIcon       *gicon,
 			   const char  *title)
 {
-	if (icon_name || stock_id)
+	if (icon_name || gicon || stock_id)
 		panel_load_menu_image_deferred (item, icon_size,
-						stock_id, icon_name, NULL);
+						stock_id, gicon,
+						icon_name, NULL);
 
 	setup_menuitem (item, icon_size, NULL, title);
 }
@@ -1825,6 +1898,7 @@ static void
 panel_load_menu_image_deferred (GtkWidget   *image_menu_item,
 				GtkIconSize  icon_size,
 				const char  *stock_id,
+				GIcon       *gicon,
 				const char  *image_filename,
 				const char  *fallback_image_filename)
 {
@@ -1845,6 +1919,10 @@ panel_load_menu_image_deferred (GtkWidget   *image_menu_item,
 	gtk_object_sink (GTK_OBJECT (image));
 
 	icon->stock_id       = stock_id;
+	if (gicon)
+		icon->gicon  = g_object_ref (gicon);
+	else
+		icon->gicon  = NULL;
 	icon->image          = g_strdup (image_filename);
 	icon->fallback_image = g_strdup (fallback_image_filename);
 	icon->icon_size      = icon_size;
